@@ -1,6 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: ascii -*-
-"""
+r"""
+==============
+ CSS Minifier
+==============
+
+CSS Minifier.
+
+The minifier is based on the semantics of the `YUI compressor`_\\, which
+itself is based on `the rule list by Isaac Schlueter`_\\.
+
 :Copyright:
 
  Copyright 2011 - 2014
@@ -20,15 +29,6 @@
  See the License for the specific language governing permissions and
  limitations under the License.
 
-==============
- CSS Minifier
-==============
-
-CSS Minifier.
-
-The minifier is based on the semantics of the `YUI compressor`_\\, which
-itself is based on `the rule list by Isaac Schlueter`_\\.
-
 This module is a re-implementation aiming for speed instead of maximum
 compression, so it can be used at runtime (rather than during a preprocessing
 step). RCSSmin does syntactical compression only (removing spaces, comments
@@ -63,22 +63,22 @@ Here's a feature list:
   - Same for ``:first-letter``
 
 rcssmin.c is a reimplementation of rcssmin.py in C and improves runtime up to
-factor 50 or so (depending on the input).
+factor 100 or so (depending on the input). docs/BENCHMARKS in the source
+distribution contains the details.
 
 Both python 2 (>= 2.4) and python 3 are supported.
 
 .. _YUI compressor: https://github.com/yui/yuicompressor/
 
-.. _the rule list by Isaac Schlueter: https://github.com/isaacs/cssmin/tree/
+.. _the rule list by Isaac Schlueter: https://github.com/isaacs/cssmin/
 """
-if 1:
+if __doc__:
     # pylint: disable = W0622
-    __doc__ = getattr(__doc__, 'decode', lambda x: __doc__)('latin-1')
-__author__ = "Andr\xe9 Malo"
-__author__ = getattr(__author__, 'decode', lambda x: __author__)('latin-1')
+    __doc__ = __doc__.encode('ascii').decode('unicode_escape')
+__author__ = r"Andr\xe9 Malo".encode('ascii').decode('unicode_escape')
 __docformat__ = "restructuredtext en"
 __license__ = "Apache License, Version 2.0"
-__version__ = '1.0.3'
+__version__ = '1.0.4'
 __all__ = ['cssmin']
 
 import re as _re
